@@ -9,9 +9,17 @@ time_features_mapping = {"year_week":"weekofyear",
                          "month_day":"day",
                          "week_day":"dayofweek"}
 
-def compute_train_features(data, ts_uid_columns, time_features, lags, window_shifts,
-                           window_sizes, window_functions, ignore_const_cols=True,
-                           n_jobs=1):
+def parse_window_functions(window_functions):
+    _window_functions = list()
+    for func_name,window_func_definition in window_functions.items():
+        func_call,window_shifts,window_sizes = window_func_definition
+        for window_shift in window_shifts:
+            for window_size in window_sizes:
+                _window_functions.append((func_name, func_call, window_shift, window_size))
+    return _window_functions
+
+def compute_train_features(data, ts_uid_columns, time_features, lags, window_functions, 
+                           ignore_const_cols=True, n_jobs=1):
     """
     Parameters
     ----------
@@ -23,12 +31,8 @@ def compute_train_features(data, ts_uid_columns, time_features, lags, window_shi
         Time attributes to include as features.
     lags: list
         List of integer lag values to include as features.
-    window_shifts: list
-        List of integer window shift values.
-    window_sizes: list
-        List of integer window sizes values to include as features.
     window_functions: list
-        List of string names of the window functions to include as features.
+       List with the definition of the rolling window functions to compute.
     ignore_const_cols: bool
         Specify whether to ignore constant columns.
     n_jobs: int
@@ -50,27 +54,30 @@ def compute_train_features(data, ts_uid_columns, time_features, lags, window_shi
         calendar_features = compute_calendar_features(**input_params)
         all_features_list.append(calendar_features)
 
-    lag_kwargs = [{"lag":lag} for lag in lags]  
-    rw_kwargs =  [{"window_shift":window_shift, "window_func":window_func, "window_size":window_size}
-                    for window_shift in window_shifts
-                    for window_func in window_functions
-                    for window_size in window_sizes]
-    input_kwargs = lag_kwargs + rw_kwargs
+    # generating the lag & rolling window features
+    if (len(lags) > 0) or (len(window_functions) > 0):
+        lag_kwargs = [{"lag":lag} for lag in lags]  
+        rw_kwargs =  [{"func_name":window_func[0],
+                       "func_call":window_func[1], 
+                       "window_shift":window_func[2], 
+                       "window_size":window_func[3]}
+                       for window_func in window_functions]
+        input_kwargs = lag_kwargs + rw_kwargs
 
-    grouped = data.loc[:, ts_uid_columns+["y"]].groupby(ts_uid_columns)["y"]
-    with Parallel(n_jobs=n_jobs) as parallel:
-        delayed_func = delayed(compute_lagged_train_feature)
-        lagged_features = parallel(delayed_func(grouped, **kwargs) for kwargs in input_kwargs)
-        lagged_features = pd.DataFrame({feature.name:feature.values for feature in lagged_features})
-        all_features_list.append(lagged_features)
+        grouped = data.loc[:, ts_uid_columns+["y"]].groupby(ts_uid_columns)["y"]
+        with Parallel(n_jobs=n_jobs) as parallel:
+            delayed_func = delayed(compute_lagged_train_feature)
+            lagged_features = parallel(delayed_func(grouped, **kwargs) for kwargs in input_kwargs)
+            lagged_features = pd.DataFrame({feature.name:feature.values for feature in lagged_features})
+            all_features_list.append(lagged_features)
 
     # merging all features
     all_features = pd.concat(all_features_list, axis=1)
     all_features.set_index(data.index, inplace=True)
     return all_features
 
-def compute_predict_features(data, ts_uid_columns, time_features, lags, window_shifts,
-                             window_sizes, window_functions, ignore_const_cols=True):
+def compute_predict_features(data, ts_uid_columns, time_features, lags, 
+                             window_functions, ignore_const_cols=True):
     """
     Parameters
     ----------
@@ -82,12 +89,8 @@ def compute_predict_features(data, ts_uid_columns, time_features, lags, window_s
         Time attributes to include as features.
     lags: list
         List of integer lag values to include as features.
-    window_shifts: list
-        List of integer window shift values.
-    window_sizes: list
-        List of integer window sizes values to include as features.
     window_functions: list
-        List of string names of the window functions to include as features.
+       List with the definition of the rolling window functions to compute.
     ignore_const_cols: bool
         Specify whether to ignore constant columns.
     Returns
@@ -113,13 +116,11 @@ def compute_predict_features(data, ts_uid_columns, time_features, lags, window_s
                                     columns=column_names)
         all_features_list.append(lag_features)
 
-    if (len(window_sizes) > 0) & (len(window_functions) > 0):
-        column_names = [f"{window_func}{window}_shift{window_shift}"
-                        for window_shift in window_shifts
-                        for window_func in window_functions
-                        for window in window_sizes]
+    if len(window_functions) > 0:
+        column_names = [f"{func_name}{window_size}_shift{window_shift}"
+                        for func_name,_,window_shift,window_size in window_functions]
         rw_features = pd.DataFrame(np.nan, index=range(len(data)),
-                                    columns=column_names)
+                                   columns=column_names)
         all_features_list.append(rw_features)
     
     # merging all features
@@ -209,58 +210,71 @@ def fill_time_gaps(data, freq="D"):
     filled_data = pd.merge(filled_data, data.drop("y", axis=1), on=["ds"], how="left")
     return filled_data
 
-def compute_lagged_train_feature(grouped, lag=None, window_shift=None, window_func=None, window_size=None):
+def compute_lagged_train_feature(grouped, lag=None, func_name=None, func_call=None, window_shift=None, window_size=None):
     """
     grouped: pandas.core.groupby.generic.SeriesGroupBy
         Groupby object containing the response variable "y"
         grouped by ts_uid_columns.
     lag: int
-        integer lag value.
+        Integer lag value.
+    func_name: string
+        Name of the rolling window function.
+    func_call: function or None
+        Callable if a custom function, None otherwise.
     window_shift: int
-        integer window shift value.
-    window_func: string
-        string names of the window function.
+        Integer window shift value.
     window_size: int
-        integer window sizes value.
+        Integer window size value.
     """
     is_lag_feature = lag is not None
-    is_rw_feature = (window_shift is not None) and (window_func is not None) and (window_size is not None)
-
+    is_rw_feature = (func_name is not None) and (window_shift is not None) and (window_size is not None)
     if is_lag_feature and not is_rw_feature:
         feature_values = grouped.shift(lag)
         feature_values.name = f"lag{lag}"
     elif is_rw_feature and not is_lag_feature:
-        feature_values = grouped.apply(lambda x: getattr(x.shift(window_shift).rolling(window_size), window_func)())
-        feature_values.name = f"{window_func}{window_size}_shift{window_shift}"
+        if func_call is None:
+            # native pandas method
+            feature_values = grouped.apply(lambda x: getattr(x.shift(window_shift).rolling(window_size), func_name)())
+        else:
+            # custom function
+            feature_values = grouped.apply(lambda x: x.shift(window_shift).rolling(window_size).apply(func_call, raw=True))
+        feature_values.name = f"{func_name}{window_size}_shift{window_shift}"
     else:
         raise ValueError("Invalid input parameters.")
 
     return feature_values
 
-def compute_lagged_predict_feature(grouped, lag=None, window_shift=None, window_func=None, window_size=None):
+def compute_lagged_predict_feature(grouped, lag=None, func_name=None, func_call=None, window_shift=None, window_size=None):
     """
     grouped: pandas.core.groupby.generic.SeriesGroupBy
         Groupby object containing the response variable "y"
         grouped by ts_uid_columns.
     lag: int
-        integer lag value.
+        Integer lag value.
+    func_name: string
+        Name of the rolling window function.
+    func_call: function or None
+        Callable if a custom function, None otherwise.
     window_shift: int
-        integer window shift value.
-    window_func: string
-        string names of the window function.
+        Integer window shift value.
     window_size: int
-        integer window sizes value.
+        Integer window size value.
     """
     is_lag_feature = lag is not None
-    is_rw_feature = (window_shift is not None) and (window_func is not None) and (window_size is not None)
+    is_rw_feature = (func_name is not None) and (window_shift is not None) and (window_size is not None)
     if is_lag_feature and not is_rw_feature:
         feature_values = grouped.apply(lambda x: x.iloc[-lag])
         feature_values.name = f"lag{lag}"
     elif is_rw_feature and not is_lag_feature:
         lidx = -(window_size + window_shift-1)
         ridx = -(window_shift-1) if window_shift > 1 else None
-        feature_values = grouped.apply(lambda x: getattr(x.iloc[lidx:ridx], window_func)())
-        feature_values.name = f"{window_func}{window_size}_shift{window_shift}"
+        if func_call is None:
+            # native pandas method
+            feature_values = grouped.apply(lambda x: getattr(x.iloc[lidx:ridx], func_name)())
+        else:
+            # custom function
+            feature_values = grouped.apply(lambda x: func_call(x.iloc[lidx:ridx].values))
+        feature_values.name = f"{func_name}{window_size}_shift{window_shift}"
     else:
         raise ValueError("Invalid input parameters.")
         
