@@ -8,7 +8,8 @@ from tsforest import metrics
 from tsforest.trend import TrendModel
 from tsforest.features import (compute_train_features, 
                                compute_predict_features,
-                               compute_lagged_predict_feature)
+                               compute_lagged_predict_feature,
+                               parse_window_functions)
 from tsforest.config import (calendar_features_names,
                              calendar_cyclical_features_names)
 from sklearn.base import TransformerMixin
@@ -25,7 +26,7 @@ AVAILABLE_TIME_FEATURES = ["year", "quarter", "month", "days_in_month",
                            "week_day_cos", "week_day_sin", "year_day_cos", "year_day_sin",
                            "year_week_cos", "year_week_sin", "month_cos", "month_sin"]
 
-# available methods in pandas.core.window.Rolling as of pandas 1.0.4 
+# available methods in pandas.core.window.Rolling as of pandas 1.0.5
 AVAILABLE_RW_FUNCTIONS = ["count", "sum", "mean", "median", "var", "std", "min", 
                           "max", "corr", "cov", "skew", "kurt", "quantile"]
 
@@ -54,12 +55,8 @@ class ForecasterBase(object):
         value, a trained sklearn.base.TransformerMixin object.
     lags: list
         List of integer lag values to include as features.
-    window_shifts: list
-        List of integer window shift values.
-    window_sizes: list
-        List of integer window sizes values to include as features.
-    window_functions: list
-        List of string names of the window functions to include as features.
+    window_functions: dict
+       Dictionary with the definition of the rolling window funtions.
     n_jobs: int
         Number of jobs to run in parallel.
     copy: bool
@@ -68,8 +65,8 @@ class ForecasterBase(object):
     """
     def __init__(self, model_params=dict(), time_features=[], exclude_features=[], 
                  categorical_features=dict(), calendar_anomaly=list(), ts_uid_columns=list(), 
-                 trend_models=dict(), target_scalers=dict(), lags=list(), window_shifts=[1], 
-                 window_sizes=list(), window_functions=list(), n_jobs=-1, copy=False):
+                 trend_models=dict(), target_scalers=dict(), lags=list(), window_functions=dict(), 
+                 n_jobs=-1, copy=False):
         self.model = None
         self.model_params = model_params
         self.time_features = time_features
@@ -84,8 +81,6 @@ class ForecasterBase(object):
         self.trend_models = trend_models
         self.target_scalers = target_scalers
         self.lags = lags
-        self.window_shifts = window_shifts
-        self.window_sizes = window_sizes
         self.window_functions = window_functions
         self.n_jobs = n_jobs
         self.copy = copy
@@ -158,30 +153,8 @@ class ForecasterBase(object):
                 raise ValueError("Values in 'lags' should be integers.")
             elif any([x<1 for x in self.lags]):
                 raise ValueError("Values in 'lags' should be integers greater or equal to 1.")
-
-        if not isinstance(self.window_shifts, list):
-            raise TypeError("Parameter 'window_shifts' should be of type 'list'.")
-        else:
-            if any([type(x)!=int for x in self.window_shifts]):
-                raise ValueError("Values in 'window_shifts' should be integers.")
-            elif any([x<1 for x in self.window_shifts]):
-                raise ValueError("Values in 'window_shifts' should be integers greater or equal to 1.")
         
-        if not isinstance(self.window_sizes, list):
-            raise TypeError("Parameter 'window_sizes' should be of type 'list'.")
-        else:
-            if any([type(x)!=int for x in self.window_sizes]):
-                raise ValueError("Values in 'window_sizes' should be integers.")
-            elif any([x<1 for x in self.window_sizes]):
-                raise ValueError("Values in 'window_sizes' should be integers greater or equal to 1.")
-        
-        if not isinstance(self.window_functions, list):
-            raise TypeError("Parameter 'window_functions' should be of type list.")
-        else:
-            if any([type(x)!=str for x in self.window_functions]):
-                raise ValueError("Values in 'window_functions' should be string names.")
-            elif any([x not in AVAILABLE_RW_FUNCTIONS for x in self.window_functions]):
-                raise ValueError(f"Values in 'window_functions' should be any of: {AVAILABLE_RW_FUNCTIONS}.")
+        # todo: add validation for window_functions dicts               
     
     def _validate_input_data(self, train_data, valid_index):
         if not isinstance(train_data, pd.DataFrame):
@@ -259,9 +232,7 @@ class ForecasterBase(object):
                                                     ts_uid_columns=self.ts_uid_columns,
                                                     time_features=self.time_features,
                                                     lags=self.lags,
-                                                    window_shifts=self.window_shifts,
-                                                    window_sizes=self.window_sizes,
-                                                    window_functions=self.window_functions,
+                                                    window_functions=self._window_functions,
                                                     ignore_const_cols=False)
 
         if "calendar_anomaly" in predict_features.columns:
@@ -322,9 +293,7 @@ class ForecasterBase(object):
                                                 ts_uid_columns=self.ts_uid_columns,
                                                 time_features=self.time_features,
                                                 lags=self.lags,
-                                                window_shifts=self.window_shifts,
-                                                window_sizes=self.window_sizes,
-                                                window_functions=self.window_functions,
+                                                window_functions=self._window_functions,
                                                 ignore_const_cols=True,
                                                 n_jobs=self.n_jobs)
         if "zero_response" in train_features.columns:
@@ -351,12 +320,18 @@ class ForecasterBase(object):
         self.raw_train_columns = list(train_data.columns)
         if self.copy:
             train_data = train_data.copy(deep=True)
+
         if len(self.ts_uid_columns) == 0:
             # adds a dummy '_internal_ts_uid' column in case of single ts data
             train_data["_internal_ts_uid"] = 0
             self.ts_uid_columns = ["_internal_ts_uid"]
         train_data = train_data.sort_values(self.ts_uid_columns+["ds"], axis=0)
         
+        if len(self.window_functions) > 0:
+            self._window_functions = parse_window_functions(self.window_functions)
+        else:
+            self._window_functions = list()
+        
         if len(self.trend_models) > 0 or len(self.target_scalers) > 0:
             train_data = self.prepare_target(train_data)
         else:
@@ -420,7 +395,8 @@ class ForecasterBase(object):
         self.model.fit(**kwargs)
         self.best_iteration = self.model.best_iteration
 
-    def predict(self, predict_data, recursive=False, return_trend=False, bias_corr_func=None):
+    def predict(self, predict_data, recursive=False, return_trend=False, 
+                bias_corr_func=None, predict_kwargs=dict()):
         """
         Parameters
         ----------
@@ -436,7 +412,9 @@ class ForecasterBase(object):
         bias_corr_func: function
             Function to perform bias correction on recursive prediction. 
             It receives a 1-dimensional array 'x' an return an 1-dimensional 
-            array of the same lenght. 
+            array of the same lenght.
+        predict_kwargs: dict
+            Extra arguments passed to the predict call of the model.
         Returns
         ----------
         prediction_dataframe: pandas.DataFrame
@@ -458,12 +436,12 @@ class ForecasterBase(object):
 
         predict_features = self._prepare_predict_features(predict_data)
         if not recursive:
-            prediction = self.model.predict(predict_features)
+            prediction = self.model.predict(predict_features, predict_kwargs)
             prediction_dataframe = (predict_data
                                     .loc[:, ["ds"]+self.ts_uid_columns]
                                     .assign(y_pred = prediction))
         else:
-            _prediction_dataframe = self.recursive_predict(predict_features, bias_corr_func)
+            _prediction_dataframe = self.recursive_predict(predict_features, bias_corr_func, predict_kwargs)
             prediction_dataframe = pd.merge(predict_data.loc[:, ["ds"]+self.ts_uid_columns],
                                             _prediction_dataframe, 
                                             how="left", on=["ds"]+self.ts_uid_columns)
@@ -499,7 +477,7 @@ class ForecasterBase(object):
         self.predict_features = predict_features
         return prediction_dataframe
                 
-    def recursive_predict(self, predict_features, bias_corr_func):
+    def recursive_predict(self, predict_features, bias_corr_func, predict_kwargs):
         min_predict_time = predict_features.ds.min()
         max_predict_time = predict_features.ds.max()
         max_train_time = self.train_data.ds.max()
@@ -511,8 +489,9 @@ class ForecasterBase(object):
             "predict time period cannot have time gaps from last time-step in training data."
 
         ts_uid_in_predict = predict_features.loc[:, self.ts_uid_columns].drop_duplicates()
-        max_offset = max(0 if len(self.lags)==0 else max(self.lags), \
-                         0 if len(self.window_sizes)==0 else max(self.window_sizes)+max(self.window_shifts))
+        max_lag = max(self.lags + [0])
+        max_rw  = max([np.sum(window_func[2:]) for window_func in self._window_functions] + [0])
+        max_offset = max(max_lag, max_rw)
         max_offset_time = min_predict_time - pd.DateOffset(max_offset+1)
         train_temp = (self.train_data
                       .loc[:, self.ts_uid_columns+["ds","y"]]
@@ -529,10 +508,11 @@ class ForecasterBase(object):
 
             for time_step in np.sort(predict_features.ds.unique()):
                 lag_kwargs = [{"lag":lag} for lag in self.lags]   
-                rw_kwargs =  [{"window_shift":window_shift, "window_func":window_func, "window_size":window_size}
-                              for window_shift in self.window_shifts
-                              for window_func in self.window_functions
-                              for window_size in self.window_sizes]
+                rw_kwargs =  [{"func_name":window_func[0],
+                               "func_call":window_func[1],
+                               "window_shift":window_func[2], 
+                               "window_size":window_func[3]}
+                              for window_func in self._window_functions]
                 input_kwargs = lag_kwargs + rw_kwargs
 
                 grouped = train_temp.groupby(self.ts_uid_columns)["y"]
@@ -540,7 +520,7 @@ class ForecasterBase(object):
                 for lagged_feature in lagged_features:
                     predict_features.loc[time_step, lagged_feature.name].loc[lagged_feature.index] = lagged_feature.values
 
-                _prediction = self.model.predict(predict_features.loc[time_step])
+                _prediction = self.model.predict(predict_features.loc[time_step], predict_kwargs)
                 if bias_corr_func is not None: 
                     _prediction = bias_corr_func(_prediction)
                 _prediction_dataframe = (predict_features
